@@ -88,26 +88,37 @@ prefs.register_preferences(
     )
 
 
-mod_support_code = ''
 typestrs = ['int', 'long', 'long long', 'float', 'double', 'long double']
 floattypestrs = ['float', 'double', 'long double']
+hightype_support_code = 'template < typename T1, typename T2 > struct _higher_type;\n'
 for ix, xtype in enumerate(typestrs):
     for iy, ytype in enumerate(typestrs):
         hightype = typestrs[max(ix, iy)]
-        if xtype in floattypestrs or ytype in floattypestrs:
-            expr = 'fmod(fmod(x, y)+y, y)'
-        else:
-            expr = '((x%y)+y)%y'
-        mod_support_code += '''
-        inline {hightype} _brian_mod({xtype} ux, {ytype} uy)
-        {{
-            const {hightype} x = ({hightype})ux;
-            const {hightype} y = ({hightype})uy;
-            return {expr};
-        }}
-        '''.format(hightype=hightype, xtype=xtype, ytype=ytype, expr=expr)
+        hightype_support_code += '''
+template < > struct _higher_type<{xtype},{ytype}> {{ typedef {hightype} type; }};
+        '''.format(hightype=hightype, xtype=xtype, ytype=ytype)
 
-_universal_support_code = deindent(mod_support_code)+'''
+mod_support_code = '''
+
+template < typename T1, typename T2 >
+static inline typename _higher_type<T1,T2>::type
+_brian_mod(T1 x, T2 y)
+{{
+    return x-y*floor(1.0*x/y);
+}}
+'''
+
+floordiv_support_code = '''
+
+template < typename T1, typename T2 >
+static inline typename _higher_type<T1,T2>::type
+_brian_floordiv(T1 x, T2 y)
+{{
+    return floor(1.0*x/y);
+}}
+'''
+
+pow_support_code = '''
 #ifdef _MSC_VER
 #define _brian_pow(x, y) (pow((double)(x), (y)))
 #else
@@ -115,6 +126,9 @@ _universal_support_code = deindent(mod_support_code)+'''
 #endif
 '''
 
+_universal_support_code = (hightype_support_code + mod_support_code +
+                           floordiv_support_code + pow_support_code)
+
 
 class CPPCodeGenerator(CodeGenerator):
     '''
@@ -221,8 +235,7 @@ class CPPCodeGenerator(CodeGenerator):
             code += ' // ' + comment
         return code
     
-    def translate_to_read_arrays(self, statements):
-        read, write, indices, conditional_write_vars = self.arrays_helper(statements)
+    def translate_to_read_arrays(self, read, write, indices):
         lines = []
         # index and read arrays (index arrays first)
         for varname in itertools.chain(indices, read):
@@ -237,8 +250,7 @@ class CPPCodeGenerator(CodeGenerator):
             lines.append(line)
         return lines
 
-    def translate_to_declarations(self, statements):
-        read, write, indices, conditional_write_vars = self.arrays_helper(statements)
+    def translate_to_declarations(self, read, write, indices):
         lines = []
         # simply declare variables that will be written but not read
         for varname in write:
@@ -248,8 +260,7 @@ class CPPCodeGenerator(CodeGenerator):
                 lines.append(line)
         return lines
 
-    def translate_to_statements(self, statements):
-        read, write, indices, conditional_write_vars = self.arrays_helper(statements)
+    def translate_to_statements(self, statements, conditional_write_vars):
         lines = []
         # the actual code
         for stmt in statements:
@@ -263,8 +274,7 @@ class CPPCodeGenerator(CodeGenerator):
                 lines.append(line)
         return lines
 
-    def translate_to_write_arrays(self, statements):
-        read, write, indices, conditional_write_vars = self.arrays_helper(statements)
+    def translate_to_write_arrays(self, write):
         lines = []
         # write arrays
         for varname in write:
@@ -275,20 +285,65 @@ class CPPCodeGenerator(CodeGenerator):
         return lines
 
     def translate_one_statement_sequence(self, statements, scalar=False):
-        # This function is refactored into four functions which perform the
-        # four necessary operations. It's done like this so that code
-        # deriving from this class can overwrite specific parts.
+        # Note that we do not call this function from
+        # `translate_statement_sequence` (which has been overwritten)
+        # It is nevertheless implemented, so that it can be called explicitly
+        # (e.g. from the GSL code generation)
+        read, write, indices, cond_write = self.arrays_helper(statements)
         lines = []
         # index and read arrays (index arrays first)
-        lines += self.translate_to_read_arrays(statements)
+        lines += self.translate_to_read_arrays(read, write, indices)
         # simply declare variables that will be written but not read
-        lines += self.translate_to_declarations(statements)
+        lines += self.translate_to_declarations(read, write, indices)
         # the actual code
-        lines += self.translate_to_statements(statements)
+        lines += self.translate_to_statements(statements, cond_write)
         # write arrays
-        lines += self.translate_to_write_arrays(statements)
-        code = '\n'.join(lines)                
-        return stripped_deindented_lines(code)
+        lines += self.translate_to_write_arrays(write)
+        return stripped_deindented_lines('\n'.join(lines))
+
+    def translate_statement_sequence(self, sc_statements, ve_statements):
+        # This function is overwritten, since we do not want to completely
+        # separate the code generation for scalar and vector code
+
+        assert set(sc_statements.keys()) == set(ve_statements.keys())
+
+        sc_code = {}
+        ve_code = {}
+
+        for block_name in sc_statements:
+            sc_block = sc_statements[block_name]
+            ve_block = ve_statements[block_name]
+            (sc_read, sc_write,
+             sc_indices, sc_cond_write) = self.arrays_helper(sc_block)
+            (ve_read, ve_write,
+             ve_indices, ve_cond_write) = self.arrays_helper(ve_block)
+            # We want to read all scalar variables that are needed in the
+            # vector code already in the scalar code, if they are not written
+            for varname in set(ve_read):
+                var = self.variables[varname]
+                if var.scalar and varname not in ve_write:
+                    sc_read.add(varname)
+                    ve_read.remove(varname)
+
+            for (code, stmts, read, write, indices,
+                 cond_write) in [(sc_code, sc_block, sc_read, sc_write,
+                                  sc_indices, sc_cond_write),
+                                 (ve_code, ve_block, ve_read, ve_write,
+                                  ve_indices, ve_cond_write)]:
+                lines = []
+                # index and read arrays (index arrays first)
+                lines += self.translate_to_read_arrays(read, write, indices)
+                # simply declare variables that will be written but not read
+                lines += self.translate_to_declarations(read, write, indices)
+                # the actual code
+                lines += self.translate_to_statements(stmts, cond_write)
+                # write arrays
+                lines += self.translate_to_write_arrays(write)
+                code[block_name] = stripped_deindented_lines('\n'.join(lines))
+
+        kwds = self.determine_keywords()
+        return sc_code, ve_code, kwds
+
 
     def denormals_to_zero_code(self):
         if self.flush_denormals:
@@ -451,30 +506,11 @@ DEFAULT_FUNCTIONS['sign'].implementations.add_implementation(CPPCodeGenerator,
                                                              name='_sign')
 
 timestep_code = '''
-// Adapted from npy_math.h and https://www.christophlassner.de/collection-of-msvc-gcc-compatibility-tricks.html
-#ifndef _BRIAN_REPLACE_ISINF_MSVC
-#define _BRIAN_REPLACE_ISINF_MSVC
-#if defined(_MSC_VER)
-#if _MSC_VER < 1900
-namespace std {
-  template <typename T>
-  bool isinf(const T &x) { return (!_finite(x))&&(!_isnan(x)); }
+static inline int64_t _timestep(double t, double dt)
+{
+    return (int64_t)((t + 1e-3*dt)/dt); 
 }
-#endif
-#endif
-#endif
-        static inline int _timestep(double t, double dt)
-        {
-            if (std::isinf(t))
-            {
-                if (t < 0)
-                    return INT_MIN;
-                else
-                    return INT_MAX;
-            }
-            return (int)((t + 1e-3*dt)/dt); 
-        }
-        '''
+'''
 DEFAULT_FUNCTIONS['timestep'].implementations.add_implementation(CPPCodeGenerator,
                                                                  code=timestep_code,
                                                                  name='_timestep')
