@@ -1,5 +1,5 @@
-#  tests for brian2-2.1.3.1-py36he980bc4_0 (this is a generated file);
-print('===== testing package: brian2-2.1.3.1-py36he980bc4_0 =====');
+#  tests for brian2-2.2-py36he980bc4_0 (this is a generated file);
+print('===== testing package: brian2-2.2-py36he980bc4_0 =====');
 print('running run_test.py');
 #  --- run_test.py (begin) ---
 # Run a simple test that uses the main simulation elements and force code
@@ -13,7 +13,7 @@ test_transmission_simple()
 
 #  --- run_test.py (end) ---
 
-print('===== brian2-2.1.3.1-py36he980bc4_0 OK =====');
+print('===== brian2-2.2-py36he980bc4_0 OK =====');
 print("import: 'brian2'")
 import brian2
 
